import { DefineSchema, RegisterSchema, SchemaRef } from '../src/decorators';
import { kSchema } from '../src/utility/kschema';
import Schema from 'schemastery';

@RegisterSchema()
class Post {
  constructor(_: Partial<Post>) {}

  @DefineSchema()
  name: string;

  getName?() {
    return this.name;
  }

  @DefineSchema({
    type: SchemaRef(() => Post),
  })
  child?: Post;
}

describe('Recurse schema', () => {
  it('should be a Schema', () => {
    expect(Post[kSchema]).toBeDefined();
  });

  it('child should be Schema', () => {
    const childPost = (Post as Schema<Post>).dict.child;
    expect(childPost[kSchema]).toBeDefined();
    expect(childPost === Post);
  });

  it('child of child should be Schema', () => {
    const childPost = (Post as Schema<Post>).dict.child.dict.child;
    expect(childPost[kSchema]).toBeDefined();
    expect(childPost === Post);
  });

  it('should resolve value', () => {
    const post = new Post({
      name: 'test',
    });
    expect(post.getName()).toBe('test');
  });

  it('should be able to recurse', () => {
    const post = new Post({
      name: 'test',
      child: {
        name: 'test',
      },
    });
    expect(post.child.getName()).toBe('test');
  });

  it('should recurse twice', () => {
    const post = new Post({
      name: 'test',
      child: {
        name: 'test',
        child: {
          name: 'test',
        },
      },
    });

    expect(post.getName()).toBe('test');
    expect(post.child.getName()).toBe('test');
    expect(post.child.child.getName()).toBe('test');
  });
});
